@@ -47,6 +47,8 @@ typedef struct _GstFlacDecClass GstFlacDecClass;
 struct _GstFlacDec {
   GstElement     element;
 
+  /* < private > */
+
 #if !defined(FLAC_API_VERSION_CURRENT) || FLAC_API_VERSION_CURRENT < 8
   FLAC__SeekableStreamDecoder *seekable_decoder; /* for pull-based operation  */
 #else
@@ -61,7 +63,6 @@ struct _GstFlacDec {
   GstPad        *srcpad;
 
   gboolean       init;
-  gboolean       need_newsegment;
 
   guint64        offset;      /* current byte offset of input */
 
@@ -71,6 +72,11 @@ struct _GstFlacDec {
 
   GstSegment     segment;     /* the currently configured segment, in
                                * samples/audio frames (DEFAULT format) */
+  gboolean       running;
+  gboolean       discont;
+  GstEvent      *close_segment;
+  GstEvent      *start_segment;
+  GstTagList    *tags;
 
   GstFlowReturn  last_flow;   /* the last flow return received from either
                                * gst_pad_push or gst_pad_buffer_alloc */
