@@ -30,6 +30,7 @@
 
 #include <gst/gsttagsetter.h>
 #include <gst/tag/tag.h>
+#include <gst/audio/audio.h>
 #include "gstspeexenc.h"
 
 GST_DEBUG_CATEGORY_STATIC (speexenc_debug);
@@ -48,7 +49,8 @@ static GstStaticPadTemplate sink_factory = GST_STATIC_PAD_TEMPLATE ("sink",
 static GstStaticPadTemplate src_factory = GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_SRC,
     GST_PAD_ALWAYS,
-    GST_STATIC_CAPS ("audio/x-speex")
+    GST_STATIC_CAPS ("audio/x-speex, "
+        "rate = (int) [ 6000, 48000 ], " "channels = (int) [ 1, 2]")
     );
 
 static const GstElementDetails speexenc_details =
@@ -59,6 +61,7 @@ GST_ELEMENT_DETAILS ("Speex audio encoder",
 
 #define DEFAULT_QUALITY         8.0
 #define DEFAULT_BITRATE         0
+#define DEFAULT_MODE            GST_SPEEX_ENC_MODE_AUTO
 #define DEFAULT_VBR             FALSE
 #define DEFAULT_ABR             0
 #define DEFAULT_VAD             FALSE
@@ -68,18 +71,38 @@ GST_ELEMENT_DETAILS ("Speex audio encoder",
 
 enum
 {
-  ARG_0,
-  ARG_QUALITY,
-  ARG_BITRATE,
-  ARG_VBR,
-  ARG_ABR,
-  ARG_VAD,
-  ARG_DTX,
-  ARG_COMPLEXITY,
-  ARG_NFRAMES,
-  ARG_LAST_MESSAGE
+  PROP_0,
+  PROP_QUALITY,
+  PROP_BITRATE,
+  PROP_MODE,
+  PROP_VBR,
+  PROP_ABR,
+  PROP_VAD,
+  PROP_DTX,
+  PROP_COMPLEXITY,
+  PROP_NFRAMES,
+  PROP_LAST_MESSAGE
 };
 
+#define GST_TYPE_SPEEX_ENC_MODE (gst_speex_enc_mode_get_type())
+static GType
+gst_speex_enc_mode_get_type (void)
+{
+  static GType speex_enc_mode_type = 0;
+  static const GEnumValue speex_enc_modes[] = {
+    {GST_SPEEX_ENC_MODE_AUTO, "Auto", "auto"},
+    {GST_SPEEX_ENC_MODE_UWB, "Ultra Wide Band", "uwb"},
+    {GST_SPEEX_ENC_MODE_WB, "Wide Band", "wb"},
+    {GST_SPEEX_ENC_MODE_NB, "Narrow Band", "nb"},
+    {0, NULL, NULL},
+  };
+  if (G_UNLIKELY (speex_enc_mode_type == 0)) {
+    speex_enc_mode_type = g_enum_register_static ("GstSpeexEncMode",
+        speex_enc_modes);
+  }
+  return speex_enc_mode_type;
+}
+
 #if 0
 static const GstFormat *
 gst_speex_enc_get_formats (GstPad * pad)
@@ -113,6 +136,8 @@ static void gst_speex_enc_set_property (GObject * object, guint prop_id,
 static GstStateChangeReturn gst_speex_enc_change_state (GstElement * element,
     GstStateChange transition);
 
+static GstFlowReturn gst_speex_enc_encode (GstSpeexEnc * enc, gboolean flush);
+
 static void
 gst_speex_enc_setup_interfaces (GType speexenc_type)
 {
@@ -151,35 +176,39 @@ gst_speex_enc_class_init (GstSpeexEncClass * klass)
   gobject_class->set_property = gst_speex_enc_set_property;
   gobject_class->get_property = gst_speex_enc_get_property;
 
-  g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_QUALITY,
+  g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_QUALITY,
       g_param_spec_float ("quality", "Quality", "Encoding quality",
           0.0, 10.0, DEFAULT_QUALITY, G_PARAM_READWRITE));
-  g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_BITRATE,
+  g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_BITRATE,
       g_param_spec_int ("bitrate", "Encoding Bit-rate",
           "Specify an encoding bit-rate (in bps). (0 = automatic)",
           0, G_MAXINT, DEFAULT_BITRATE, G_PARAM_READWRITE));
-  g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_VBR,
+  g_object_class_install_property (gobject_class, PROP_MODE,
+      g_param_spec_enum ("mode", "Mode", "The encoding mode",
+          GST_TYPE_SPEEX_ENC_MODE, GST_SPEEX_ENC_MODE_AUTO,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+  g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_VBR,
       g_param_spec_boolean ("vbr", "VBR",
           "Enable variable bit-rate", DEFAULT_VBR, G_PARAM_READWRITE));
-  g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_ABR,
+  g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_ABR,
       g_param_spec_int ("abr", "ABR",
           "Enable average bit-rate (0 = disabled)",
           0, G_MAXINT, DEFAULT_ABR, G_PARAM_READWRITE));
-  g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_VAD,
+  g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_VAD,
       g_param_spec_boolean ("vad", "VAD",
           "Enable voice activity detection", DEFAULT_VAD, G_PARAM_READWRITE));
-  g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_DTX,
+  g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_DTX,
       g_param_spec_boolean ("dtx", "DTX",
           "Enable discontinuous transmission", DEFAULT_DTX, G_PARAM_READWRITE));
-  g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_COMPLEXITY,
+  g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_COMPLEXITY,
       g_param_spec_int ("complexity", "Complexity",
           "Set encoding complexity",
           0, G_MAXINT, DEFAULT_COMPLEXITY, G_PARAM_READWRITE));
-  g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_NFRAMES,
+  g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_NFRAMES,
       g_param_spec_int ("nframes", "NFrames",
           "Number of frames per buffer",
           0, G_MAXINT, DEFAULT_NFRAMES, G_PARAM_READWRITE));
-  g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_LAST_MESSAGE,
+  g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_LAST_MESSAGE,
       g_param_spec_string ("last-message", "last-message",
           "The last status message", NULL, G_PARAM_READABLE));
 
@@ -220,6 +249,39 @@ gst_speex_enc_sink_setcaps (GstPad * pad, GstCaps * caps)
   return enc->setup;
 }
 
+
+static GstCaps *
+gst_speex_enc_sink_getcaps (GstPad * pad)
+{
+  GstCaps *caps = gst_caps_copy (gst_pad_get_pad_template_caps (pad));
+  GstCaps *peercaps = NULL;
+  GstSpeexEnc *enc = GST_SPEEX_ENC (gst_pad_get_parent_element (pad));
+
+  peercaps = gst_pad_peer_get_caps (enc->srcpad);
+
+  if (peercaps) {
+    if (!gst_caps_is_empty (peercaps) && !gst_caps_is_any (peercaps)) {
+      GstStructure *ps = gst_caps_get_structure (peercaps, 0);
+      GstStructure *s = gst_caps_get_structure (caps, 0);
+      gint rate, channels;
+
+      if (gst_structure_get_int (ps, "rate", &rate)) {
+        gst_structure_fixate_field_nearest_int (s, "rate", rate);
+      }
+
+      if (gst_structure_get_int (ps, "channels", &channels)) {
+        gst_structure_fixate_field_nearest_int (s, "channels", channels);
+      }
+    }
+    gst_caps_unref (peercaps);
+  }
+
+  gst_object_unref (enc);
+
+  return caps;
+}
+
+
 static gboolean
 gst_speex_enc_convert_src (GstPad * pad, GstFormat src_format, gint64 src_value,
     GstFormat * dest_format, gint64 * dest_value)
@@ -326,6 +388,12 @@ gst_speex_enc_convert_sink (GstPad * pad, GstFormat src_format,
   return res;
 }
 
+static gint64
+gst_speex_enc_get_latency (GstSpeexEnc * enc)
+{
+  return 30 * GST_MSECOND;
+}
+
 static const GstQueryType *
 gst_speex_enc_get_query_types (GstPad * pad)
 {
@@ -333,6 +401,7 @@ gst_speex_enc_get_query_types (GstPad * pad)
     GST_QUERY_POSITION,
     GST_QUERY_DURATION,
     GST_QUERY_CONVERT,
+    GST_QUERY_LATENCY,
     0
   };
 
@@ -404,8 +473,28 @@ gst_speex_enc_src_query (GstPad * pad, GstQuery * query)
       gst_query_set_convert (query, src_fmt, src_val, dest_fmt, dest_val);
       break;
     }
+    case GST_QUERY_LATENCY:
+    {
+      gboolean live;
+      GstClockTime min_latency, max_latency;
+      gint64 latency;
+
+      if ((res = gst_pad_peer_query (pad, query))) {
+        gst_query_parse_latency (query, &live, &min_latency, &max_latency);
+
+        latency = gst_speex_enc_get_latency (enc);
+
+        /* add our latency */
+        min_latency += latency;
+        if (max_latency != -1)
+          max_latency += latency;
+
+        gst_query_set_latency (query, live, min_latency, max_latency);
+      }
+      break;
+    }
     default:
-      res = gst_pad_query_default (pad, query);
+      res = gst_pad_peer_query (pad, query);
       break;
   }
 
@@ -458,6 +547,8 @@ gst_speex_enc_init (GstSpeexEnc * enc, GstSpeexEncClass * klass)
       GST_DEBUG_FUNCPTR (gst_speex_enc_chain));
   gst_pad_set_setcaps_function (enc->sinkpad,
       GST_DEBUG_FUNCPTR (gst_speex_enc_sink_setcaps));
+  gst_pad_set_getcaps_function (enc->sinkpad,
+      GST_DEBUG_FUNCPTR (gst_speex_enc_sink_getcaps));
   gst_pad_set_query_function (enc->sinkpad,
       GST_DEBUG_FUNCPTR (gst_speex_enc_sink_query));
 
@@ -473,6 +564,7 @@ gst_speex_enc_init (GstSpeexEnc * enc, GstSpeexEncClass * klass)
 
   enc->quality = DEFAULT_QUALITY;
   enc->bitrate = DEFAULT_BITRATE;
+  enc->mode = DEFAULT_MODE;
   enc->vbr = DEFAULT_VBR;
   enc->abr = DEFAULT_ABR;
   enc->vad = DEFAULT_VAD;
@@ -480,8 +572,6 @@ gst_speex_enc_init (GstSpeexEnc * enc, GstSpeexEncClass * klass)
   enc->complexity = DEFAULT_COMPLEXITY;
   enc->nframes = DEFAULT_NFRAMES;
 
-  /* FIXME: what about enc->mode? */
-
   enc->setup = FALSE;
   enc->header_sent = FALSE;
 
@@ -534,22 +624,27 @@ gst_speex_enc_setup (GstSpeexEnc * enc)
 
   switch (enc->mode) {
     case GST_SPEEX_ENC_MODE_UWB:
+      GST_LOG_OBJECT (enc, "configuring for requested UWB mode");
       enc->speex_mode = (SpeexMode *) & speex_uwb_mode;
       break;
     case GST_SPEEX_ENC_MODE_WB:
+      GST_LOG_OBJECT (enc, "configuring for requested WB mode");
       enc->speex_mode = (SpeexMode *) & speex_wb_mode;
       break;
     case GST_SPEEX_ENC_MODE_NB:
+      GST_LOG_OBJECT (enc, "configuring for requested NB mode");
       enc->speex_mode = (SpeexMode *) & speex_nb_mode;
       break;
     case GST_SPEEX_ENC_MODE_AUTO:
       /* fall through */
+      GST_LOG_OBJECT (enc, "finding best mode");
     default:
       break;
   }
 
   if (enc->rate > 25000) {
     if (enc->mode == GST_SPEEX_ENC_MODE_AUTO) {
+      GST_LOG_OBJECT (enc, "selected UWB mode for samplerate %d", enc->rate);
       enc->speex_mode = (SpeexMode *) & speex_uwb_mode;
     } else {
       if (enc->speex_mode != &speex_uwb_mode) {
@@ -559,6 +654,7 @@ gst_speex_enc_setup (GstSpeexEnc * enc)
     }
   } else if (enc->rate > 12500) {
     if (enc->mode == GST_SPEEX_ENC_MODE_AUTO) {
+      GST_LOG_OBJECT (enc, "selected WB mode for samplerate %d", enc->rate);
       enc->speex_mode = (SpeexMode *) & speex_wb_mode;
     } else {
       if (enc->speex_mode != &speex_wb_mode) {
@@ -568,6 +664,7 @@ gst_speex_enc_setup (GstSpeexEnc * enc)
     }
   } else {
     if (enc->mode == GST_SPEEX_ENC_MODE_AUTO) {
+      GST_LOG_OBJECT (enc, "selected NB mode for samplerate %d", enc->rate);
       enc->speex_mode = (SpeexMode *) & speex_nb_mode;
     } else {
       if (enc->speex_mode != &speex_nb_mode) {
@@ -638,6 +735,9 @@ gst_speex_enc_setup (GstSpeexEnc * enc)
 
   speex_encoder_ctl (enc->state, SPEEX_GET_LOOKAHEAD, &enc->lookahead);
 
+  GST_LOG_OBJECT (enc, "we have frame size %d, lookahead %d", enc->frame_size,
+      enc->lookahead);
+
   enc->setup = TRUE;
 
   return TRUE;
@@ -664,10 +764,15 @@ gst_speex_enc_buffer_from_data (GstSpeexEnc * enc, guchar * data,
 static GstFlowReturn
 gst_speex_enc_push_buffer (GstSpeexEnc * enc, GstBuffer * buffer)
 {
-  enc->bytes_out += GST_BUFFER_SIZE (buffer);
+  guint size;
+
+  size = GST_BUFFER_SIZE (buffer);
+
+  enc->bytes_out += size;
+
+  GST_DEBUG_OBJECT (enc, "pushing output buffer of size %u", size);
 
   return gst_pad_push (enc->srcpad, buffer);
-
 }
 
 static GstCaps *
@@ -720,7 +825,7 @@ gst_speex_enc_sinkevent (GstPad * pad, GstEvent * event)
 
   switch (GST_EVENT_TYPE (event)) {
     case GST_EVENT_EOS:
-      enc->eos = TRUE;
+      gst_speex_enc_encode (enc, TRUE);
       res = gst_pad_event_default (pad, event);
       break;
     case GST_EVENT_TAG:
@@ -745,6 +850,81 @@ gst_speex_enc_sinkevent (GstPad * pad, GstEvent * event)
   return res;
 }
 
+static GstFlowReturn
+gst_speex_enc_encode (GstSpeexEnc * enc, gboolean flush)
+{
+  gint frame_size = enc->frame_size;
+  gint bytes = frame_size * 2 * enc->channels;
+  GstFlowReturn ret = GST_FLOW_OK;
+
+  if (flush && gst_adapter_available (enc->adapter) % bytes != 0) {
+    guint diff = gst_adapter_available (enc->adapter) % bytes;
+    GstBuffer *buf = gst_buffer_new_and_alloc (diff);
+
+    memset (GST_BUFFER_DATA (buf), 0, diff);
+    gst_adapter_push (enc->adapter, buf);
+  }
+
+  while (gst_adapter_available (enc->adapter) >= bytes) {
+    gint16 *data;
+    gint outsize, written;
+    GstBuffer *outbuf;
+
+    data = (gint16 *) gst_adapter_take (enc->adapter, bytes);
+
+    enc->samples_in += frame_size;
+
+    GST_DEBUG_OBJECT (enc, "encoding %d samples (%d bytes)", frame_size, bytes);
+
+    if (enc->channels == 2) {
+      speex_encode_stereo_int (data, frame_size, &enc->bits);
+    }
+    speex_encode_int (enc->state, data, &enc->bits);
+
+    g_free (data);
+
+    enc->frameno++;
+    enc->frameno_out++;
+
+    if ((enc->frameno % enc->nframes) != 0)
+      continue;
+
+    speex_bits_insert_terminator (&enc->bits);
+    outsize = speex_bits_nbytes (&enc->bits);
+
+    ret = gst_pad_alloc_buffer_and_set_caps (enc->srcpad,
+        GST_BUFFER_OFFSET_NONE, outsize, GST_PAD_CAPS (enc->srcpad), &outbuf);
+
+    if ((GST_FLOW_OK != ret))
+      goto done;
+
+    written = speex_bits_write (&enc->bits,
+        (gchar *) GST_BUFFER_DATA (outbuf), outsize);
+    g_assert (written == outsize);
+    speex_bits_reset (&enc->bits);
+
+    GST_BUFFER_TIMESTAMP (outbuf) = enc->start_ts +
+        gst_util_uint64_scale_int ((enc->frameno_out - 1) * frame_size -
+        enc->lookahead, GST_SECOND, enc->rate);
+    GST_BUFFER_DURATION (outbuf) = gst_util_uint64_scale_int (frame_size,
+        GST_SECOND, enc->rate);
+    /* set gp time and granulepos; see gst-plugins-base/ext/ogg/README */
+    GST_BUFFER_OFFSET_END (outbuf) = enc->granulepos_offset +
+        ((enc->frameno_out) * frame_size - enc->lookahead);
+    GST_BUFFER_OFFSET (outbuf) =
+        gst_util_uint64_scale_int (GST_BUFFER_OFFSET_END (outbuf), GST_SECOND,
+        enc->rate);
+
+    ret = gst_speex_enc_push_buffer (enc, outbuf);
+
+    if ((GST_FLOW_OK != ret) && (GST_FLOW_NOT_LINKED != ret))
+      goto done;
+  }
+
+done:
+
+  return ret;
+}
 
 static GstFlowReturn
 gst_speex_enc_chain (GstPad * pad, GstBuffer * buf)
@@ -781,8 +961,12 @@ gst_speex_enc_chain (GstPad * pad, GstBuffer * buf)
     caps = gst_pad_get_caps (enc->srcpad);
     caps = gst_speex_enc_set_header_on_caps (caps, buf1, buf2);
 
+    gst_caps_set_simple (caps,
+        "rate", G_TYPE_INT, enc->rate,
+        "channels", G_TYPE_INT, enc->channels, NULL);
+
     /* negotiate with these caps */
-    GST_DEBUG ("here are the caps: %" GST_PTR_FORMAT, caps);
+    GST_DEBUG_OBJECT (enc, "here are the caps: %" GST_PTR_FORMAT, caps);
     gst_pad_set_caps (enc->srcpad, caps);
 
     gst_buffer_set_caps (buf1, caps);
@@ -807,72 +991,80 @@ gst_speex_enc_chain (GstPad * pad, GstBuffer * buf)
     enc->header_sent = TRUE;
   }
 
-  {
-    gint frame_size = enc->frame_size;
-    gint bytes = frame_size * 2 * enc->channels;
-
-    /* push buffer to adapter */
-    gst_adapter_push (enc->adapter, buf);
-    buf = NULL;
-
-    while (gst_adapter_available (enc->adapter) >= bytes) {
-      gint16 *data;
-      gint i;
-      gint outsize, written;
-      GstBuffer *outbuf;
-
-      data = (gint16 *) gst_adapter_peek (enc->adapter, bytes);
-
-      for (i = 0; i < frame_size * enc->channels; i++) {
-        enc->input[i] = (gfloat) data[i];
-      }
-      gst_adapter_flush (enc->adapter, bytes);
-
-      enc->samples_in += frame_size;
-
-      if (enc->channels == 2) {
-        speex_encode_stereo (enc->input, frame_size, &enc->bits);
-      }
-      speex_encode (enc->state, enc->input, &enc->bits);
-
-      enc->frameno++;
-
-      if ((enc->frameno % enc->nframes) != 0)
-        continue;
-
-      speex_bits_insert_terminator (&enc->bits);
-      outsize = speex_bits_nbytes (&enc->bits);
-
-      ret = gst_pad_alloc_buffer_and_set_caps (enc->srcpad,
-          GST_BUFFER_OFFSET_NONE, outsize, GST_PAD_CAPS (enc->srcpad), &outbuf);
-
-      if ((GST_FLOW_OK != ret))
-        goto done;
-
-      written = speex_bits_write (&enc->bits,
-          (gchar *) GST_BUFFER_DATA (outbuf), outsize);
-      g_assert (written == outsize);
-      speex_bits_reset (&enc->bits);
-
-      GST_BUFFER_TIMESTAMP (outbuf) =
-          gst_util_uint64_scale_int (enc->frameno * frame_size -
-          enc->lookahead, GST_SECOND, enc->rate);
-      GST_BUFFER_DURATION (outbuf) = gst_util_uint64_scale_int (frame_size,
-          GST_SECOND, enc->rate);
-      /* set gp time and granulepos; see gst-plugins-base/ext/ogg/README */
-      GST_BUFFER_OFFSET_END (outbuf) =
-          ((enc->frameno + 1) * frame_size - enc->lookahead);
-      GST_BUFFER_OFFSET (outbuf) =
-          gst_util_uint64_scale_int (GST_BUFFER_OFFSET_END (outbuf), GST_SECOND,
-          enc->rate);
-
-      ret = gst_speex_enc_push_buffer (enc, outbuf);
-
-      if ((GST_FLOW_OK != ret) && (GST_FLOW_NOT_LINKED != ret))
-        goto done;
+  /* Save the timestamp of the first buffer. This will be later
+   * used as offset for all following buffers */
+  if (enc->start_ts == GST_CLOCK_TIME_NONE) {
+    if (GST_BUFFER_TIMESTAMP_IS_VALID (buf)) {
+      enc->start_ts = GST_BUFFER_TIMESTAMP (buf);
+      enc->granulepos_offset = gst_util_uint64_scale
+          (GST_BUFFER_TIMESTAMP (buf), enc->rate, GST_SECOND);
+    } else {
+      enc->start_ts = 0;
+      enc->granulepos_offset = 0;
     }
   }
 
+  /* Check if we have a continous stream, if not drop some samples or the buffer or
+   * insert some silence samples */
+  if (enc->next_ts != GST_CLOCK_TIME_NONE &&
+      GST_BUFFER_TIMESTAMP (buf) < enc->next_ts) {
+    guint64 diff = enc->next_ts - GST_BUFFER_TIMESTAMP (buf);
+    guint64 diff_bytes;
+
+    GST_WARNING_OBJECT (enc, "Buffer is older than previous "
+        "timestamp + duration (%" GST_TIME_FORMAT "< %" GST_TIME_FORMAT
+        "), cannot handle. Clipping buffer.",
+        GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (buf)),
+        GST_TIME_ARGS (enc->next_ts));
+
+    diff_bytes = GST_CLOCK_TIME_TO_FRAMES (diff, enc->rate) * enc->channels * 2;
+    if (diff_bytes >= GST_BUFFER_SIZE (buf)) {
+      gst_buffer_unref (buf);
+      return GST_FLOW_OK;
+    }
+    buf = gst_buffer_make_metadata_writable (buf);
+    GST_BUFFER_DATA (buf) += diff_bytes;
+    GST_BUFFER_SIZE (buf) -= diff_bytes;
+
+    GST_BUFFER_TIMESTAMP (buf) += diff;
+    if (GST_BUFFER_DURATION_IS_VALID (buf))
+      GST_BUFFER_DURATION (buf) -= diff;
+  }
+
+  if (enc->next_ts != GST_CLOCK_TIME_NONE
+      && GST_BUFFER_TIMESTAMP_IS_VALID (buf)) {
+    guint64 max_diff =
+        gst_util_uint64_scale (enc->frame_size, GST_SECOND, enc->rate);
+
+    if (GST_BUFFER_TIMESTAMP (buf) != enc->next_ts &&
+        GST_BUFFER_TIMESTAMP (buf) - enc->next_ts > max_diff) {
+      GST_WARNING_OBJECT (enc,
+          "Discontinuity detected: %" G_GUINT64_FORMAT " > %" G_GUINT64_FORMAT,
+          GST_BUFFER_TIMESTAMP (buf) - enc->next_ts, max_diff);
+
+      gst_speex_enc_encode (enc, TRUE);
+
+      enc->frameno_out = 0;
+      enc->start_ts = GST_BUFFER_TIMESTAMP (buf);
+      enc->granulepos_offset = gst_util_uint64_scale
+          (GST_BUFFER_TIMESTAMP (buf), enc->rate, GST_SECOND);
+    }
+  }
+
+  if (GST_BUFFER_TIMESTAMP_IS_VALID (buf)
+      && GST_BUFFER_DURATION_IS_VALID (buf))
+    enc->next_ts = GST_BUFFER_TIMESTAMP (buf) + GST_BUFFER_DURATION (buf);
+  else
+    enc->next_ts = GST_CLOCK_TIME_NONE;
+
+  GST_DEBUG_OBJECT (enc, "received buffer of %u bytes", GST_BUFFER_SIZE (buf));
+
+  /* push buffer to adapter */
+  gst_adapter_push (enc->adapter, buf);
+  buf = NULL;
+
+  ret = gst_speex_enc_encode (enc, FALSE);
+
 done:
 
   if (buf)
@@ -883,7 +1075,6 @@ done:
   /* ERRORS */
 not_setup:
   {
-    gst_buffer_unref (buf);
     GST_ELEMENT_ERROR (enc, CORE, NEGOTIATION, (NULL),
         ("encoder not initialized (input is not audio?)"));
     ret = GST_FLOW_NOT_NEGOTIATED;
@@ -902,31 +1093,34 @@ gst_speex_enc_get_property (GObject * object, guint prop_id, GValue * value,
   enc = GST_SPEEX_ENC (object);
 
   switch (prop_id) {
-    case ARG_QUALITY:
+    case PROP_QUALITY:
       g_value_set_float (value, enc->quality);
       break;
-    case ARG_BITRATE:
+    case PROP_BITRATE:
       g_value_set_int (value, enc->bitrate);
       break;
-    case ARG_VBR:
+    case PROP_MODE:
+      g_value_set_enum (value, enc->mode);
+      break;
+    case PROP_VBR:
       g_value_set_boolean (value, enc->vbr);
       break;
-    case ARG_ABR:
+    case PROP_ABR:
       g_value_set_int (value, enc->abr);
       break;
-    case ARG_VAD:
+    case PROP_VAD:
       g_value_set_boolean (value, enc->vad);
       break;
-    case ARG_DTX:
+    case PROP_DTX:
       g_value_set_boolean (value, enc->dtx);
       break;
-    case ARG_COMPLEXITY:
+    case PROP_COMPLEXITY:
       g_value_set_int (value, enc->complexity);
       break;
-    case ARG_NFRAMES:
+    case PROP_NFRAMES:
       g_value_set_int (value, enc->nframes);
       break;
-    case ARG_LAST_MESSAGE:
+    case PROP_LAST_MESSAGE:
       g_value_set_string (value, enc->last_message);
       break;
     default:
@@ -944,28 +1138,31 @@ gst_speex_enc_set_property (GObject * object, guint prop_id,
   enc = GST_SPEEX_ENC (object);
 
   switch (prop_id) {
-    case ARG_QUALITY:
+    case PROP_QUALITY:
       enc->quality = g_value_get_float (value);
       break;
-    case ARG_BITRATE:
+    case PROP_BITRATE:
       enc->bitrate = g_value_get_int (value);
       break;
-    case ARG_VBR:
+    case PROP_MODE:
+      enc->mode = g_value_get_enum (value);
+      break;
+    case PROP_VBR:
       enc->vbr = g_value_get_boolean (value);
       break;
-    case ARG_ABR:
+    case PROP_ABR:
       enc->abr = g_value_get_int (value);
       break;
-    case ARG_VAD:
+    case PROP_VAD:
       enc->vad = g_value_get_boolean (value);
       break;
-    case ARG_DTX:
+    case PROP_DTX:
       enc->dtx = g_value_get_boolean (value);
       break;
-    case ARG_COMPLEXITY:
+    case PROP_COMPLEXITY:
       enc->complexity = g_value_get_int (value);
       break;
-    case ARG_NFRAMES:
+    case PROP_NFRAMES:
       enc->nframes = g_value_get_int (value);
       break;
     default:
@@ -987,7 +1184,11 @@ gst_speex_enc_change_state (GstElement * element, GstStateChange transition)
     case GST_STATE_CHANGE_READY_TO_PAUSED:
       speex_bits_init (&enc->bits);
       enc->frameno = 0;
+      enc->frameno_out = 0;
       enc->samples_in = 0;
+      enc->start_ts = GST_CLOCK_TIME_NONE;
+      enc->next_ts = GST_CLOCK_TIME_NONE;
+      enc->granulepos_offset = 0;
       break;
     case GST_STATE_CHANGE_PAUSED_TO_PLAYING:
       /* fall through */
