@@ -43,69 +43,11 @@
 #include "config.h"
 #endif
 
-#include <gst/gst.h>
-#include <gst/base/gstcollectpads.h>
-
-#include <string.h>
+#include "multipartmux.h"
 
 GST_DEBUG_CATEGORY_STATIC (gst_multipart_mux_debug);
 #define GST_CAT_DEFAULT gst_multipart_mux_debug
 
-#define GST_TYPE_MULTIPART_MUX (gst_multipart_mux_get_type())
-#define GST_MULTIPART_MUX(obj) (G_TYPE_CHECK_INSTANCE_CAST((obj),GST_TYPE_MULTIPART_MUX, GstMultipartMux))
-#define GST_MULTIPART_MUX_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST((klass),GST_TYPE_MULTIPART_MUX, GstMultipartMux))
-#define GST_MULTIPART_MUX_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GST_TYPE_MULTIPART_MUX, GstMultipartMuxClass))
-#define GST_IS_MULTIPART_MUX(obj) (G_TYPE_CHECK_INSTANCE_TYPE((obj),GST_TYPE_MULTIPART_MUX))
-#define GST_IS_MULTIPART_MUX_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE((klass),GST_TYPE_MULTIPART_MUX))
-
-typedef struct _GstMultipartMux GstMultipartMux;
-typedef struct _GstMultipartMuxClass GstMultipartMuxClass;
-
-/* all information needed for one multipart stream */
-typedef struct
-{
-  GstCollectData collect;       /* we extend the CollectData */
-
-  GstBuffer *buffer;            /* the queued buffer for this pad */
-  GstClockTime timestamp;       /* its timestamp, converted to running_time so that we can
-                                   correctly sort over multiple segments. */
-}
-GstMultipartPad;
-
-/**
- * GstMultipartMux:
- *
- * The opaque #GstMultipartMux structure.
- */
-struct _GstMultipartMux
-{
-  GstElement element;
-
-  /* pad */
-  GstPad *srcpad;
-
-  /* sinkpads */
-  GstCollectPads *collect;
-
-  gint numpads;
-
-  /* offset in stream */
-  guint64 offset;
-
-  /* boundary string */
-  gchar *boundary;
-
-  gboolean negotiated;
-  gboolean need_segment;
-};
-
-struct _GstMultipartMuxClass
-{
-  GstElementClass parent_class;
-
-  GHashTable *mimetypes;
-};
-
 /* elementfactory information */
 static const GstElementDetails gst_multipart_mux_details =
 GST_ELEMENT_DETAILS ("Multipart muxer",
@@ -349,6 +291,9 @@ gst_multipart_mux_get_mime (GstMultipartMux * mux, GstStructure * s)
   GstMultipartMuxClass *klass;
   const gchar *mime;
   const gchar *name;
+  gint rate;
+  gint channels;
+  gint bitrate = 0;
 
   klass = GST_MULTIPART_MUX_GET_CLASS (mux);
 
@@ -357,8 +302,38 @@ gst_multipart_mux_get_mime (GstMultipartMux * mux, GstStructure * s)
   /* use hashtable to convert to mime type */
   mime = g_hash_table_lookup (klass->mimetypes, name);
   if (mime == NULL) {
-    /* no mime type mapping, use name */
-    mime = name;
+    if (!strcmp (name, "audio/x-adpcm"))
+      gst_structure_get_int (s, "bitrate", &bitrate);
+
+    switch (bitrate) {
+      case 16000:
+        mime = "audio/G726-16";
+        break;
+      case 24000:
+        mime = "audio/G726-24";
+        break;
+      case 32000:
+        mime = "audio/G726-32";
+        break;
+      case 40000:
+        mime = "audio/G726-40";
+        break;
+      default:
+        /* no mime type mapping, use name */
+        mime = name;
+        break;
+    }
+  }
+  /* RFC2046 requires audio/basic to be mulaw 8000Hz mono */
+  if (g_ascii_strcasecmp (mime, "audio/basic") == 0) {
+    if (gst_structure_get_int (s, "rate", &rate) &&
+        gst_structure_get_int (s, "channels", &channels)) {
+      if (rate != 8000 || channels != 1) {
+        mime = name;
+      }
+    } else {
+      mime = name;
+    }
   }
   return mime;
 }
@@ -485,9 +460,12 @@ gst_multipart_mux_collected (GstCollectPads * pads, GstMultipartMux * mux)
     newcaps = gst_caps_new_simple ("multipart/x-mixed-replace",
         "boundary", G_TYPE_STRING, mux->boundary, NULL);
 
-    if (!gst_pad_set_caps (mux->srcpad, newcaps))
+    if (!gst_pad_set_caps (mux->srcpad, newcaps)) {
+      gst_caps_unref (newcaps);
       goto nego_error;
+    }
 
+    gst_caps_unref (newcaps);
     mux->negotiated = TRUE;
   }
 
