@@ -52,11 +52,13 @@
 #include "gstrtpL16depay.h"
 #include "gstrtpL16pay.h"
 #include "gstasteriskh263.h"
+#include "gstrtpmp1sdepay.h"
 #include "gstrtpmp2tdepay.h"
 #include "gstrtpmp2tpay.h"
 #include "gstrtpmp4vdepay.h"
 #include "gstrtpmp4vpay.h"
 #include "gstrtpmp4adepay.h"
+#include "gstrtpmp4apay.h"
 #include "gstrtpmp4gdepay.h"
 #include "gstrtpmp4gpay.h"
 #include "gstrtpspeexpay.h"
@@ -165,6 +167,9 @@ plugin_init (GstPlugin * plugin)
   if (!gst_asteriskh263_plugin_init (plugin))
     return FALSE;
 
+  if (!gst_rtp_mp1s_depay_plugin_init (plugin))
+    return FALSE;
+
   if (!gst_rtp_mp2t_depay_plugin_init (plugin))
     return FALSE;
 
@@ -177,6 +182,9 @@ plugin_init (GstPlugin * plugin)
   if (!gst_rtp_mp4v_depay_plugin_init (plugin))
     return FALSE;
 
+  if (!gst_rtp_mp4a_pay_plugin_init (plugin))
+    return FALSE;
+
   if (!gst_rtp_mp4a_depay_plugin_init (plugin))
     return FALSE;
 
