@@ -142,47 +142,48 @@ gst_wavparse_class_init (GstWavParseClass * klass)
 }
 
 static void
-gst_wavparse_dispose (GObject * object)
+gst_wavparse_reset (GstWavParse * wav)
 {
-  GstWavParse *wav;
+  wav->state = GST_WAVPARSE_START;
 
-  GST_DEBUG ("WAV: Dispose");
-  wav = GST_WAVPARSE (object);
+  /* These will all be set correctly in the fmt chunk */
+  wav->depth = 0;
+  wav->rate = 0;
+  wav->width = 0;
+  wav->channels = 0;
+  wav->blockalign = 0;
+  wav->bps = 0;
+  wav->fact = 0;
+  wav->offset = 0;
+  wav->end_offset = 0;
+  wav->dataleft = 0;
+  wav->datasize = 0;
+  wav->datastart = 0;
+  wav->duration = 0;
+  wav->got_fmt = FALSE;
+  wav->first = TRUE;
 
+  if (wav->seek_event)
+    gst_event_unref (wav->seek_event);
+  wav->seek_event = NULL;
   if (wav->adapter) {
-    g_object_unref (wav->adapter);
+    gst_adapter_clear (wav->adapter);
     wav->adapter = NULL;
   }
-
-  G_OBJECT_CLASS (parent_class)->dispose (object);
+  if (wav->tags)
+    gst_tag_list_free (wav->tags);
+  wav->tags = NULL;
 }
 
 static void
-gst_wavparse_reset (GstWavParse * wavparse)
+gst_wavparse_dispose (GObject * object)
 {
-  wavparse->state = GST_WAVPARSE_START;
+  GstWavParse *wav = GST_WAVPARSE (object);
 
-  /* These will all be set correctly in the fmt chunk */
-  wavparse->depth = 0;
-  wavparse->rate = 0;
-  wavparse->width = 0;
-  wavparse->channels = 0;
-  wavparse->blockalign = 0;
-  wavparse->bps = 0;
-  wavparse->fact = 0;
-  wavparse->offset = 0;
-  wavparse->end_offset = 0;
-  wavparse->dataleft = 0;
-  wavparse->datasize = 0;
-  wavparse->datastart = 0;
-  wavparse->got_fmt = FALSE;
-  wavparse->first = TRUE;
+  GST_DEBUG_OBJECT (wav, "WAV: Dispose");
+  gst_wavparse_reset (wav);
 
-  if (wavparse->seek_event)
-    gst_event_unref (wavparse->seek_event);
-  wavparse->seek_event = NULL;
-  if (wavparse->adapter)
-    gst_adapter_clear (wavparse->adapter);
+  G_OBJECT_CLASS (parent_class)->dispose (object);
 }
 
 static void
@@ -243,9 +244,21 @@ gst_wavparse_create_sourcepad (GstWavParse * wavparse)
 #define uint64_scale_modulo(val, nom, denom) \
   ((val % denom) * (nom % denom) % denom)
 
-/* Like gst_util_uint64_scale_int, but performs ceiling division. */
+/* Like gst_util_uint64_scale, but performs ceiling division. */
 static guint64
 uint64_ceiling_scale_int (guint64 val, gint num, gint denom)
+{
+  guint64 result = gst_util_uint64_scale (val, num, denom);
+
+  if (uint64_scale_modulo (val, num, denom) == 0)
+    return result;
+  else
+    return result + 1;
+}
+
+/* Like gst_util_uint64_scale, but performs ceiling division. */
+static guint64
+uint64_ceiling_scale (guint64 val, guint64 num, guint64 denom)
 {
   guint64 result = gst_util_uint64_scale_int (val, num, denom);
 
@@ -255,6 +268,8 @@ uint64_ceiling_scale_int (guint64 val, gint num, gint denom)
     return result + 1;
 }
 
+
+/* FIXME: why is that not in use? */
 #if 0
 static void
 gst_wavparse_parse_adtl (GstWavParse * wavparse, int len)
@@ -676,6 +691,7 @@ gst_wavparse_other (GstWavParse * wav)
 }
 #endif
 
+
 static gboolean
 gst_wavparse_parse_file_header (GstElement * element, GstBuffer * buf)
 {
@@ -739,8 +755,6 @@ gst_wavparse_perform_seek (GstWavParse * wav, GstEvent * event)
   gint64 last_stop;
 
   if (event) {
-    GstFormat fmt;
-
     GST_DEBUG_OBJECT (wav, "doing seek with event");
 
     gst_event_parse_seek (event, &rate, &format, &flags,
@@ -750,24 +764,28 @@ gst_wavparse_perform_seek (GstWavParse * wav, GstEvent * event)
     if (rate < 0.0)
       goto negative_rate;
 
-    fmt = wav->segment.format;
-
-    /* we have to have a format as the segment format. Try to convert
-     * if not. */
     if (format != wav->segment.format) {
+      GST_INFO_OBJECT (wav, "converting seek-event from %s to %s",
+          gst_format_get_name (format),
+          gst_format_get_name (wav->segment.format));
       res = TRUE;
       if (cur_type != GST_SEEK_TYPE_NONE)
-        res = gst_pad_query_convert (wav->srcpad, format, cur, &fmt, &cur);
+        res =
+            gst_pad_query_convert (wav->srcpad, format, cur,
+            &wav->segment.format, &cur);
       if (res && stop_type != GST_SEEK_TYPE_NONE)
-        res = gst_pad_query_convert (wav->srcpad, format, stop, &fmt, &stop);
+        res =
+            gst_pad_query_convert (wav->srcpad, format, stop,
+            &wav->segment.format, &stop);
       if (!res)
         goto no_format;
 
-      format = fmt;
+      format = wav->segment.format;
     }
   } else {
     GST_DEBUG_OBJECT (wav, "doing seek without event");
     flags = 0;
+    rate = 1.0;
     cur_type = GST_SEEK_TYPE_SET;
     stop_type = GST_SEEK_TYPE_SET;
   }
@@ -821,11 +839,16 @@ gst_wavparse_perform_seek (GstWavParse * wav, GstEvent * event)
     /* bring offset to bytes, if the bps is 0, we have the segment in BYTES and
      * we can just copy the last_stop. If not, we use the bps to convert TIME to
      * bytes. */
-    if (wav->bps)
+    if (wav->bps > 0)
       wav->offset =
-          uint64_ceiling_scale_int (seeksegment.last_stop, wav->bps,
+          uint64_ceiling_scale (seeksegment.last_stop, (guint64) wav->bps,
           GST_SECOND);
-    else
+    else if (wav->fact) {
+      guint64 bps =
+          gst_util_uint64_scale_int (wav->datasize, wav->rate, wav->fact);
+      wav->offset =
+          uint64_ceiling_scale (seeksegment.last_stop, bps, GST_SECOND);
+    } else
       wav->offset = seeksegment.last_stop;
     GST_LOG_OBJECT (wav, "offset=%" G_GUINT64_FORMAT, wav->offset);
     wav->offset -= (wav->offset % wav->bytes_per_sample);
@@ -838,9 +861,14 @@ gst_wavparse_perform_seek (GstWavParse * wav, GstEvent * event)
   }
 
   if (stop_type != GST_SEEK_TYPE_NONE) {
-    if (wav->bps)
-      wav->end_offset = uint64_ceiling_scale_int (stop, wav->bps, GST_SECOND);
-    else
+    if (wav->bps > 0)
+      wav->end_offset =
+          uint64_ceiling_scale (stop, (guint64) wav->bps, GST_SECOND);
+    else if (wav->fact) {
+      guint64 bps =
+          gst_util_uint64_scale_int (wav->datasize, wav->rate, wav->fact);
+      wav->end_offset = uint64_ceiling_scale (stop, bps, GST_SECOND);
+    } else
       wav->end_offset = stop;
     GST_LOG_OBJECT (wav, "end_offset=%" G_GUINT64_FORMAT, wav->end_offset);
     wav->end_offset -= (wav->end_offset % wav->bytes_per_sample);
@@ -863,9 +891,9 @@ gst_wavparse_perform_seek (GstWavParse * wav, GstEvent * event)
   wav->dataleft = wav->end_offset - wav->offset;
 
   GST_DEBUG_OBJECT (wav,
-      "seek: offset %" G_GUINT64_FORMAT ", end %" G_GUINT64_FORMAT ", segment %"
-      GST_TIME_FORMAT " -- %" GST_TIME_FORMAT, wav->offset, wav->end_offset,
-      GST_TIME_ARGS (seeksegment.start), GST_TIME_ARGS (stop));
+      "seek: rate %lf, offset %" G_GUINT64_FORMAT ", end %" G_GUINT64_FORMAT
+      ", segment %" GST_TIME_FORMAT " -- %" GST_TIME_FORMAT, rate, wav->offset,
+      wav->end_offset, GST_TIME_ARGS (seeksegment.start), GST_TIME_ARGS (stop));
 
   /* prepare for streaming again */
   if (wav->srcpad) {
@@ -951,7 +979,7 @@ no_format:
  *
  * Peek next chunk info (tag and size)
  *
- * Returns: %TRUE when one chunk info has been got from the adapter
+ * Returns: %TRUE when the chunk info (header) is available
  */
 static gboolean
 gst_wavparse_peek_chunk_info (GstWavParse * wav, guint32 * tag, guint32 * size)
@@ -979,7 +1007,7 @@ gst_wavparse_peek_chunk_info (GstWavParse * wav, guint32 * tag, guint32 * size)
  *
  * Peek enough data for one full chunk
  *
- * Returns: %TRUE when one chunk has been got
+ * Returns: %TRUE when the full chunk is available
  */
 static gboolean
 gst_wavparse_peek_chunk (GstWavParse * wav, guint32 * tag, guint32 * size)
@@ -1002,19 +1030,55 @@ gst_wavparse_peek_chunk (GstWavParse * wav, guint32 * tag, guint32 * size)
   }
 }
 
+/*
+ * gst_wavparse_calculate_duration:
+ * @wav: wavparse object
+ *
+ * Calculate duration on demand and store in @wav. Prefer bps, but use fact as a
+ * fallback.
+ *
+ * Returns: %TRUE if duration is available.
+ */
 static gboolean
-gst_wavparse_get_upstream_size (GstWavParse * wav, gint64 * len)
+gst_wavparse_calculate_duration (GstWavParse * wav)
 {
-  gboolean res = FALSE;
-  GstFormat fmt = GST_FORMAT_BYTES;
-  GstPad *peer;
+  if (wav->duration > 0)
+    return TRUE;
 
-  if ((peer = gst_pad_get_peer (wav->sinkpad))) {
-    res = gst_pad_query_duration (peer, &fmt, len);
-    gst_object_unref (peer);
+  if (wav->bps > 0) {
+    wav->duration =
+        uint64_ceiling_scale (wav->datasize, GST_SECOND, (guint64) wav->bps);
+    GST_INFO_OBJECT (wav, "Got duration (bps) %" GST_TIME_FORMAT,
+        GST_TIME_ARGS (wav->duration));
+    return TRUE;
+  } else if (wav->fact) {
+    wav->duration = uint64_ceiling_scale_int (GST_SECOND, wav->fact, wav->rate);
+    GST_INFO_OBJECT (wav, "Got duration (fact) %" GST_TIME_FORMAT,
+        GST_TIME_ARGS (wav->duration));
+    return TRUE;
   }
+  return FALSE;
+}
 
-  return res;
+static void
+gst_waveparse_ignore_chunk (GstWavParse * wav, GstBuffer * buf, guint32 tag,
+    guint32 size)
+{
+  guint flush;
+
+  if (wav->streaming) {
+    if (!gst_wavparse_peek_chunk (wav, &tag, &size))
+      return;
+  }
+  GST_DEBUG_OBJECT (wav, "Ignoring tag %" GST_FOURCC_FORMAT,
+      GST_FOURCC_ARGS (tag));
+  flush = 8 + ((size + 1) & ~1);
+  wav->offset += flush;
+  if (wav->streaming) {
+    gst_adapter_flush (wav->adapter, flush);
+  } else {
+    gst_buffer_unref (buf);
+  }
 }
 
 static GstFlowReturn
@@ -1026,10 +1090,12 @@ gst_wavparse_stream_headers (GstWavParse * wav)
   guint32 tag, size;
   gboolean gotdata = FALSE;
   GstCaps *caps;
-  gint64 duration;
   gchar *codec_name = NULL;
   GstEvent **event_p;
+  GstFormat bformat;
+  gint64 upstream_size = 0;
 
+  /* search for "_fmt" chunk, which should be first */
   while (!wav->got_fmt) {
     GstBuffer *extra;
 
@@ -1097,6 +1163,7 @@ gst_wavparse_stream_headers (GstWavParse * wav)
     wav->vbr = FALSE;
 
     g_free (header);
+    header = NULL;
 
     /* do format specific handling */
     switch (wav->format) {
@@ -1162,10 +1229,12 @@ gst_wavparse_stream_headers (GstWavParse * wav)
 
   }
 
+  bformat = GST_FORMAT_BYTES;
+  gst_pad_query_peer_duration (wav->sinkpad, &bformat, &upstream_size);
+  GST_DEBUG_OBJECT (wav, "upstream size %" G_GUINT64_FORMAT, upstream_size);
+
   /* loop headers until we get data */
   while (!gotdata) {
-    gint64 upstream_size = 0;
-
     if (wav->streaming) {
       if (!gst_wavparse_peek_chunk_info (wav, &tag, &size))
         return GST_FLOW_OK;
@@ -1178,16 +1247,14 @@ gst_wavparse_stream_headers (GstWavParse * wav)
       size = GST_READ_UINT32_LE (GST_BUFFER_DATA (buf) + 4);
     }
 
-    GST_DEBUG_OBJECT (wav, "Got TAG: %" GST_FOURCC_FORMAT,
-        GST_FOURCC_ARGS (tag));
-
-    gst_wavparse_get_upstream_size (wav, &upstream_size);
+    GST_INFO_OBJECT (wav,
+        "Got TAG: %" GST_FOURCC_FORMAT ", offset %" G_GUINT64_FORMAT,
+        GST_FOURCC_ARGS (tag), wav->offset);
 
     /* wav is a st00pid format, we don't know for sure where data starts.
      * So we have to go bit by bit until we find the 'data' header
      */
     switch (tag) {
-        /* TODO : Implement the various cases */
       case GST_RIFF_TAG_data:{
         GstFormat fmt;
 
@@ -1216,43 +1283,140 @@ gst_wavparse_stream_headers (GstWavParse * wav)
         break;
       }
       case GST_RIFF_TAG_fact:{
-        /* number of samples (for compressed formats) */
-        if (wav->streaming) {
-          const guint8 *data = NULL;
+        if (wav->format != GST_RIFF_WAVE_FORMAT_MPEGL12 &&
+            wav->format != GST_RIFF_WAVE_FORMAT_MPEGL3) {
+          const guint data_size = 4;
 
-          if (gst_adapter_available (wav->adapter) < 8 + 4) {
+          /* number of samples (for compressed formats) */
+          if (wav->streaming) {
+            const guint8 *data = NULL;
+
+            if (gst_adapter_available (wav->adapter) < 8 + data_size) {
+              return GST_FLOW_OK;
+            }
+            gst_adapter_flush (wav->adapter, 8);
+            data = gst_adapter_peek (wav->adapter, data_size);
+            wav->fact = GST_READ_UINT32_LE (data);
+            gst_adapter_flush (wav->adapter, data_size);
+          } else {
+            gst_buffer_unref (buf);
+            if ((res =
+                    gst_pad_pull_range (wav->sinkpad, wav->offset + 8,
+                        data_size, &buf)) != GST_FLOW_OK)
+              goto header_read_error;
+            wav->fact = GST_READ_UINT32_LE (GST_BUFFER_DATA (buf));
+            gst_buffer_unref (buf);
+          }
+          GST_DEBUG_OBJECT (wav, "have fact %u", wav->fact);
+          wav->offset += 8 + data_size;
+          break;
+        } else {
+          gst_waveparse_ignore_chunk (wav, buf, tag, size);
+        }
+        break;
+      }
+      case GST_RIFF_TAG_acid:{
+        const gst_riff_acid *acid = NULL;
+        const guint data_size = sizeof (gst_riff_acid);
+
+        if (wav->streaming) {
+          if (gst_adapter_available (wav->adapter) < 8 + data_size) {
             return GST_FLOW_OK;
           }
           gst_adapter_flush (wav->adapter, 8);
-          data = gst_adapter_peek (wav->adapter, 4);
-          wav->fact = GST_READ_UINT32_LE (data);
-          gst_adapter_flush (wav->adapter, 4);
+          acid = (const gst_riff_acid *) gst_adapter_peek (wav->adapter,
+              data_size);
         } else {
           gst_buffer_unref (buf);
           if ((res =
-                  gst_pad_pull_range (wav->sinkpad, wav->offset + 8, 4,
+                  gst_pad_pull_range (wav->sinkpad, wav->offset + 8,
+                      data_size, &buf)) != GST_FLOW_OK)
+            goto header_read_error;
+          acid = (const gst_riff_acid *) GST_BUFFER_DATA (buf);
+        }
+        GST_INFO_OBJECT (wav, "Have acid chunk");
+        /* send data as tags */
+        if (!wav->tags)
+          wav->tags = gst_tag_list_new ();
+        gst_tag_list_add (wav->tags, GST_TAG_MERGE_REPLACE,
+            GST_TAG_BEATS_PER_MINUTE, acid->tempo, NULL);
+
+        if (wav->streaming) {
+          gst_adapter_flush (wav->adapter, data_size);
+        } else {
+          gst_buffer_unref (buf);
+          wav->offset += 8 + data_size;
+        }
+        break;
+      }
+        /* FIXME: all list tags after data are ignored in streaming mode */
+      case GST_RIFF_TAG_LIST:{
+        guint32 ltag;
+
+        if (wav->streaming) {
+          const guint8 *data = NULL;
+
+          if (gst_adapter_available (wav->adapter) < 12) {
+            return GST_FLOW_OK;
+          }
+          data = gst_adapter_peek (wav->adapter, 12);
+          ltag = GST_READ_UINT32_LE (data + 8);
+        } else {
+          gst_buffer_unref (buf);
+          if ((res =
+                  gst_pad_pull_range (wav->sinkpad, wav->offset, 12,
                       &buf)) != GST_FLOW_OK)
             goto header_read_error;
-          wav->fact = GST_READ_UINT32_LE (GST_BUFFER_DATA (buf));
-          gst_buffer_unref (buf);
+          ltag = GST_READ_UINT32_LE (GST_BUFFER_DATA (buf) + 8);
+        }
+        switch (ltag) {
+          case GST_RIFF_LIST_INFO:{
+            const guint data_size = size - 12;
+            GstTagList *new;
+
+            GST_INFO_OBJECT (wav, "Have LIST chunk INFO");
+            if (wav->streaming) {
+              gst_adapter_flush (wav->adapter, 12);
+              if (gst_adapter_available (wav->adapter) < data_size) {
+                return GST_FLOW_OK;
+              }
+              gst_buffer_unref (buf);
+              buf = gst_adapter_take_buffer (wav->adapter, data_size);
+            } else {
+              wav->offset += 12;
+              gst_buffer_unref (buf);
+              if ((res =
+                      gst_pad_pull_range (wav->sinkpad, wav->offset, data_size,
+                          &buf)) != GST_FLOW_OK)
+                goto header_read_error;
+            }
+            /* parse tags */
+            gst_riff_parse_info (GST_ELEMENT (wav), buf, &new);
+            if (new) {
+              GstTagList *old = wav->tags;
+              wav->tags = gst_tag_list_merge (old, new, GST_TAG_MERGE_REPLACE);
+              if (old)
+                gst_tag_list_free (old);
+              gst_tag_list_free (new);
+            }
+            if (wav->streaming) {
+              gst_adapter_flush (wav->adapter, data_size);
+            } else {
+              gst_buffer_unref (buf);
+              wav->offset += data_size;
+            }
+            break;
+          }
+          default:
+            GST_INFO_OBJECT (wav, "Ignoring LIST chunk %" GST_FOURCC_FORMAT,
+                GST_FOURCC_ARGS (ltag));
+            gst_waveparse_ignore_chunk (wav, buf, tag, size);
+            break;
         }
-        GST_DEBUG_OBJECT (wav, "have fact %u", wav->fact);
-        wav->offset += 8 + 4;
         break;
       }
       default:
-        if (wav->streaming) {
-          if (!gst_wavparse_peek_chunk (wav, &tag, &size))
-            return GST_FLOW_OK;
-        }
-        GST_DEBUG_OBJECT (wav, "Ignoring tag %" GST_FOURCC_FORMAT,
-            GST_FOURCC_ARGS (tag));
-        wav->offset += 8 + ((size + 1) & ~1);
-        if (wav->streaming) {
-          gst_adapter_flush (wav->adapter, 8 + ((size + 1) & ~1));
-        } else {
-          gst_buffer_unref (buf);
-        }
+        gst_waveparse_ignore_chunk (wav, buf, tag, size);
     }
 
     if (upstream_size && (wav->offset >= upstream_size)) {
@@ -1264,19 +1428,19 @@ gst_wavparse_stream_headers (GstWavParse * wav)
   GST_DEBUG_OBJECT (wav, "Finished parsing headers");
 
   if (wav->bps <= 0 && wav->fact) {
+#if 0
+    /* not a good idea, as for embedded mp2/mp3 we set bps to 0 earlier */
     wav->bps =
         (guint32) gst_util_uint64_scale ((guint64) wav->rate, wav->datasize,
         (guint64) wav->fact);
-    GST_DEBUG_OBJECT (wav, "calculated bps : %d", wav->bps);
+    GST_INFO_OBJECT (wav, "calculated bps : %d, enabling VBR", wav->bps);
+#endif
     wav->vbr = TRUE;
   }
 
-  if (wav->bps > 0) {
-    duration = uint64_ceiling_scale_int (wav->datasize, GST_SECOND, wav->bps);
-    GST_DEBUG_OBJECT (wav, "Got duration %" GST_TIME_FORMAT,
-        GST_TIME_ARGS (duration));
+  if (gst_wavparse_calculate_duration (wav)) {
     gst_segment_init (&wav->segment, GST_FORMAT_TIME);
-    gst_segment_set_duration (&wav->segment, GST_FORMAT_TIME, duration);
+    gst_segment_set_duration (&wav->segment, GST_FORMAT_TIME, wav->duration);
   } else {
     /* no bitrate, let downstream peer do the math, we'll feed it bytes. */
     gst_segment_init (&wav->segment, GST_FORMAT_BYTES);
@@ -1334,8 +1498,7 @@ invalid_blockalign:
   {
     GST_ELEMENT_ERROR (wav, STREAM, FAILED, (NULL),
         ("Stream claims blockalign = %u, which is more than %u - invalid data",
-            header->blockalign,
-            header->channels * (guint) ceil (header->size / 8.0)));
+            wav->blockalign, wav->channels * (guint) ceil (wav->depth / 8.0)));
     g_free (codec_name);
     return GST_FLOW_ERROR;
   }
@@ -1563,23 +1726,21 @@ iterate_adapter:
   GST_BUFFER_OFFSET_END (buf) = nextpos / wav->bytes_per_sample;
 
   if (wav->bps > 0) {
-    /* and timestamps if we have a bitrate, be carefull for overflows */
-    timestamp = uint64_ceiling_scale_int (pos, GST_SECOND, wav->bps);
-    next_timestamp = uint64_ceiling_scale_int (nextpos, GST_SECOND, wav->bps);
+    /* and timestamps if we have a bitrate, be careful for overflows */
+    timestamp = uint64_ceiling_scale (pos, GST_SECOND, (guint64) wav->bps);
+    next_timestamp =
+        uint64_ceiling_scale (nextpos, GST_SECOND, (guint64) wav->bps);
     duration = next_timestamp - timestamp;
 
     /* update current running segment position */
     gst_segment_set_last_stop (&wav->segment, GST_FORMAT_TIME, next_timestamp);
-
-    if (wav->discont) {
-      GST_DEBUG_OBJECT (wav, "marking DISCONT");
-      GST_BUFFER_FLAG_SET (buf, GST_BUFFER_FLAG_DISCONT);
-      wav->discont = FALSE;
-    } else if (wav->vbr) {
-      /* don't set timestamps for VBR files if it's not the first buffer */
-      timestamp = GST_CLOCK_TIME_NONE;
-      duration = GST_CLOCK_TIME_NONE;
-    }
+  } else if (wav->fact) {
+    guint64 bps =
+        gst_util_uint64_scale_int (wav->datasize, wav->rate, wav->fact);
+    /* and timestamps if we have a bitrate, be careful for overflows */
+    timestamp = uint64_ceiling_scale (pos, GST_SECOND, bps);
+    next_timestamp = uint64_ceiling_scale (nextpos, GST_SECOND, bps);
+    duration = next_timestamp - timestamp;
   } else {
     /* no bitrate, all we know is that the first sample has timestamp 0, all
      * other positions and durations have unknown timestamp. */
@@ -1591,6 +1752,16 @@ iterate_adapter:
     /* update current running segment position with byte offset */
     gst_segment_set_last_stop (&wav->segment, GST_FORMAT_BYTES, nextpos);
   }
+  if ((pos > 0) && wav->vbr) {
+    /* don't set timestamps for VBR files if it's not the first buffer */
+    timestamp = GST_CLOCK_TIME_NONE;
+    duration = GST_CLOCK_TIME_NONE;
+  }
+  if (wav->discont) {
+    GST_DEBUG_OBJECT (wav, "marking DISCONT");
+    GST_BUFFER_FLAG_SET (buf, GST_BUFFER_FLAG_DISCONT);
+    wav->discont = FALSE;
+  }
 
   GST_BUFFER_TIMESTAMP (buf) = timestamp;
   GST_BUFFER_DURATION (buf) = duration;
@@ -1607,11 +1778,12 @@ iterate_adapter:
     goto push_error;
 
   if (obtained < wav->dataleft) {
+    wav->offset += obtained;
     wav->dataleft -= obtained;
   } else {
+    wav->offset += wav->dataleft;
     wav->dataleft = 0;
   }
-  wav->offset += obtained;
 
   /* Iterate until need more data, so adapter size won't grow */
   if (wav->streaming) {
@@ -1641,8 +1813,10 @@ pull_error:
   }
 push_error:
   {
-    GST_WARNING_OBJECT (wav, "Error pushing on srcpad %p, is linked? = %d",
-        wav->srcpad, gst_pad_is_linked (wav->srcpad));
+    GST_INFO_OBJECT (wav,
+        "Error pushing on srcpad %s:%s, reason %s, is linked? = %d",
+        GST_DEBUG_PAD_NAME (wav->srcpad), gst_flow_get_name (res),
+        gst_pad_is_linked (wav->srcpad));
     return res;
   }
 }
@@ -1657,7 +1831,7 @@ gst_wavparse_loop (GstPad * pad)
 
   switch (wav->state) {
     case GST_WAVPARSE_START:
-      GST_DEBUG_OBJECT (wav, "GST_WAVPARSE_START");
+      GST_INFO_OBJECT (wav, "GST_WAVPARSE_START");
       if ((ret = gst_wavparse_stream_init (wav)) != GST_FLOW_OK)
         goto pause;
 
@@ -1665,11 +1839,12 @@ gst_wavparse_loop (GstPad * pad)
       /* fall-through */
 
     case GST_WAVPARSE_HEADER:
-      GST_DEBUG_OBJECT (wav, "GST_WAVPARSE_HEADER");
+      GST_INFO_OBJECT (wav, "GST_WAVPARSE_HEADER");
       if ((ret = gst_wavparse_stream_headers (wav)) != GST_FLOW_OK)
         goto pause;
 
       wav->state = GST_WAVPARSE_DATA;
+      GST_INFO_OBJECT (wav, "GST_WAVPARSE_DATA");
       /* fall-through */
 
     case GST_WAVPARSE_DATA:
@@ -1737,7 +1912,7 @@ gst_wavparse_chain (GstPad * pad, GstBuffer * buf)
 
   switch (wav->state) {
     case GST_WAVPARSE_START:
-      GST_DEBUG_OBJECT (wav, "GST_WAVPARSE_START");
+      GST_INFO_OBJECT (wav, "GST_WAVPARSE_START");
       if ((ret = gst_wavparse_parse_stream_init (wav)) != GST_FLOW_OK)
         goto done;
 
@@ -1746,7 +1921,7 @@ gst_wavparse_chain (GstPad * pad, GstBuffer * buf)
 
       /* otherwise fall-through */
     case GST_WAVPARSE_HEADER:
-      GST_DEBUG_OBJECT (wav, "GST_WAVPARSE_HEADER");
+      GST_INFO_OBJECT (wav, "GST_WAVPARSE_HEADER");
       if ((ret = gst_wavparse_stream_headers (wav)) != GST_FLOW_OK)
         goto done;
 
@@ -1754,6 +1929,7 @@ gst_wavparse_chain (GstPad * pad, GstBuffer * buf)
         break;
 
       wav->state = GST_WAVPARSE_DATA;
+      GST_INFO_OBJECT (wav, "GST_WAVPARSE_DATA");
 
       /* fall-through */
     case GST_WAVPARSE_DATA:
@@ -1761,7 +1937,7 @@ gst_wavparse_chain (GstPad * pad, GstBuffer * buf)
         goto done;
       break;
     default:
-      g_assert_not_reached ();
+      g_return_val_if_reached (GST_FLOW_ERROR);
   }
 done:
   return ret;
@@ -1791,15 +1967,18 @@ gst_wavparse_pad_convert (GstPad * pad,
   GstWavParse *wavparse;
   gboolean res = TRUE;
 
-  wavparse = GST_WAVPARSE (gst_pad_get_parent (pad));
+  wavparse = GST_WAVPARSE (GST_PAD_PARENT (pad));
 
   if (*dest_format == src_format) {
     *dest_value = src_value;
     return TRUE;
   }
 
-  if (wavparse->bps == 0)
-    goto no_bps;
+  if ((wavparse->bps == 0) && !wavparse->fact)
+    goto no_bps_fact;
+
+  GST_INFO_OBJECT (wavparse, "converting value from %s to %s",
+      gst_format_get_name (src_format), gst_format_get_name (*dest_format));
 
   switch (src_format) {
     case GST_FORMAT_BYTES:
@@ -1810,8 +1989,21 @@ gst_wavparse_pad_convert (GstPad * pad,
           *dest_value -= *dest_value % wavparse->bytes_per_sample;
           break;
         case GST_FORMAT_TIME:
-          *dest_value =
-              gst_util_uint64_scale_int (src_value, GST_SECOND, wavparse->bps);
+          /* src_value + datastart = offset */
+          GST_INFO_OBJECT (wavparse,
+              "src=%" G_GINT64_FORMAT ", offset=%" G_GINT64_FORMAT, src_value,
+              wavparse->offset);
+          if (wavparse->bps > 0)
+            *dest_value = uint64_ceiling_scale (src_value, GST_SECOND,
+                (guint64) wavparse->bps);
+          else if (wavparse->fact) {
+            guint64 bps = uint64_ceiling_scale_int (wavparse->datasize,
+                wavparse->rate, wavparse->fact);
+
+            *dest_value = uint64_ceiling_scale_int (src_value, GST_SECOND, bps);
+          } else {
+            res = FALSE;
+          }
           break;
         default:
           res = FALSE;
@@ -1825,8 +2017,8 @@ gst_wavparse_pad_convert (GstPad * pad,
           *dest_value = src_value * wavparse->bytes_per_sample;
           break;
         case GST_FORMAT_TIME:
-          *dest_value =
-              gst_util_uint64_scale_int (src_value, GST_SECOND, wavparse->rate);
+          *dest_value = gst_util_uint64_scale (src_value, GST_SECOND,
+              (guint64) wavparse->rate);
           break;
         default:
           res = FALSE;
@@ -1837,14 +2029,21 @@ gst_wavparse_pad_convert (GstPad * pad,
     case GST_FORMAT_TIME:
       switch (*dest_format) {
         case GST_FORMAT_BYTES:
+          if (wavparse->bps > 0)
+            *dest_value = gst_util_uint64_scale (src_value,
+                (guint64) wavparse->bps, GST_SECOND);
+          else {
+            guint64 bps = gst_util_uint64_scale_int (wavparse->datasize,
+                wavparse->rate, wavparse->fact);
+
+            *dest_value = gst_util_uint64_scale (src_value, bps, GST_SECOND);
+          }
           /* make sure we end up on a sample boundary */
-          *dest_value =
-              gst_util_uint64_scale_int (src_value, wavparse->bps, GST_SECOND);
           *dest_value -= *dest_value % wavparse->blockalign;
           break;
         case GST_FORMAT_DEFAULT:
-          *dest_value =
-              gst_util_uint64_scale_int (src_value, wavparse->rate, GST_SECOND);
+          *dest_value = gst_util_uint64_scale (src_value,
+              (guint64) wavparse->rate, GST_SECOND);
           break;
         default:
           res = FALSE;
@@ -1858,14 +2057,12 @@ gst_wavparse_pad_convert (GstPad * pad,
   }
 
 done:
-  gst_object_unref (wavparse);
-
   return res;
 
   /* ERRORS */
-no_bps:
+no_bps_fact:
   {
-    GST_DEBUG_OBJECT (wavparse, "bps 0, cannot convert");
+    GST_DEBUG_OBJECT (wavparse, "bps 0 or no fact chunk, cannot convert");
     res = FALSE;
     goto done;
   }
@@ -1878,6 +2075,7 @@ gst_wavparse_get_query_types (GstPad * pad)
     GST_QUERY_POSITION,
     GST_QUERY_DURATION,
     GST_QUERY_CONVERT,
+    GST_QUERY_SEEKING,
     0
   };
 
@@ -1889,11 +2087,13 @@ static gboolean
 gst_wavparse_pad_query (GstPad * pad, GstQuery * query)
 {
   gboolean res = TRUE;
-  GstWavParse *wav = GST_WAVPARSE (GST_PAD_PARENT (pad));
+  GstWavParse *wav = GST_WAVPARSE (gst_pad_get_parent (pad));
 
   /* only if we know */
-  if (wav->state != GST_WAVPARSE_DATA)
+  if (wav->state != GST_WAVPARSE_DATA) {
+    gst_object_unref (wav);
     return FALSE;
+  }
 
   switch (GST_QUERY_TYPE (query)) {
     case GST_QUERY_POSITION:
@@ -1902,13 +2102,14 @@ gst_wavparse_pad_query (GstPad * pad, GstQuery * query)
       gint64 cur;
       GstFormat format;
 
+      /* this is not very precise, as we have pushed severla buffer upstream for prerolling */
       curb = wav->offset - wav->datastart;
       gst_query_parse_position (query, &format, NULL);
+      GST_INFO_OBJECT (wav, "pos query at %" G_GINT64_FORMAT, curb);
 
       switch (format) {
         case GST_FORMAT_TIME:
-          res &=
-              gst_wavparse_pad_convert (pad, GST_FORMAT_BYTES, curb,
+          res = gst_wavparse_pad_convert (pad, GST_FORMAT_BYTES, curb,
               &format, &cur);
           break;
         default:
@@ -1922,31 +2123,24 @@ gst_wavparse_pad_query (GstPad * pad, GstQuery * query)
     }
     case GST_QUERY_DURATION:
     {
-      gint64 endb;
-      gint64 end;
+      gint64 duration = 0;
       GstFormat format;
 
-      endb = wav->datasize;
       gst_query_parse_duration (query, &format, NULL);
 
       switch (format) {
         case GST_FORMAT_TIME:{
-          if (wav->fact) {
-            end = GST_SECOND * wav->fact / wav->rate;
-          } else {
-            res &=
-                gst_wavparse_pad_convert (pad, GST_FORMAT_BYTES, endb,
-                &format, &end);
+          if ((res = gst_wavparse_calculate_duration (wav))) {
+            duration = wav->duration;
           }
           break;
         }
         default:
           format = GST_FORMAT_BYTES;
-          end = endb;
+          duration = wav->datasize;
           break;
       }
-      if (res)
-        gst_query_set_duration (query, format, end);
+      gst_query_set_duration (query, format, duration);
       break;
     }
     case GST_QUERY_CONVERT:
@@ -1956,44 +2150,59 @@ gst_wavparse_pad_query (GstPad * pad, GstQuery * query)
 
       gst_query_parse_convert (query, &srcformat, &srcvalue,
           &dstformat, &dstvalue);
-      res &=
-          gst_wavparse_pad_convert (pad, srcformat, srcvalue,
+      res = gst_wavparse_pad_convert (pad, srcformat, srcvalue,
           &dstformat, &dstvalue);
       if (res)
         gst_query_set_convert (query, srcformat, srcvalue, dstformat, dstvalue);
       break;
     }
+    case GST_QUERY_SEEKING:{
+      GstFormat fmt;
+
+      gst_query_parse_seeking (query, &fmt, NULL, NULL, NULL);
+      if (fmt == GST_FORMAT_TIME) {
+        gboolean seekable = TRUE;
+
+        if ((wav->bps == 0) && !wav->fact) {
+          seekable = FALSE;
+        } else if (!gst_wavparse_calculate_duration (wav)) {
+          seekable = FALSE;
+        }
+        gst_query_set_seeking (query, GST_FORMAT_TIME, seekable,
+            0, wav->duration);
+        res = TRUE;
+      }
+      break;
+    }
     default:
       res = gst_pad_query_default (pad, query);
       break;
   }
+  gst_object_unref (wav);
   return res;
 }
 
 static gboolean
 gst_wavparse_srcpad_event (GstPad * pad, GstEvent * event)
 {
-  GstWavParse *wavparse = GST_WAVPARSE (GST_PAD_PARENT (pad));
-  gboolean res = TRUE;
+  GstWavParse *wavparse = GST_WAVPARSE (gst_pad_get_parent (pad));
+  gboolean res = FALSE;
 
-  GST_DEBUG_OBJECT (wavparse, "event %d, %s", GST_EVENT_TYPE (event),
-      GST_EVENT_TYPE_NAME (event));
-
-  /* can only handle events when we are in the data state */
-  if (wavparse->state != GST_WAVPARSE_DATA)
-    return FALSE;
+  GST_DEBUG_OBJECT (wavparse, "%s event", GST_EVENT_TYPE_NAME (event));
 
   switch (GST_EVENT_TYPE (event)) {
     case GST_EVENT_SEEK:
-    {
-      res = gst_wavparse_perform_seek (wavparse, event);
+      /* can only handle events when we are in the data state */
+      if (wavparse->state == GST_WAVPARSE_DATA) {
+        res = gst_wavparse_perform_seek (wavparse, event);
+      }
       gst_event_unref (event);
       break;
-    }
     default:
       res = gst_pad_push_event (wavparse->sinkpad, event);
       break;
   }
+  gst_object_unref (wavparse);
   return res;
 }
 
@@ -2025,20 +2234,17 @@ gst_wavparse_sink_activate (GstPad * sinkpad)
 static gboolean
 gst_wavparse_sink_activate_pull (GstPad * sinkpad, gboolean active)
 {
-  GstWavParse *wav = GST_WAVPARSE (gst_pad_get_parent (sinkpad));
-
-  GST_DEBUG_OBJECT (wav, "activating pull");
+  GstWavParse *wav = GST_WAVPARSE (GST_OBJECT_PARENT (sinkpad));
 
   if (active) {
     /* if we have a scheduler we can start the task */
     wav->segment_running = TRUE;
-    gst_pad_start_task (sinkpad, (GstTaskFunction) gst_wavparse_loop, sinkpad);
+    return gst_pad_start_task (sinkpad, (GstTaskFunction) gst_wavparse_loop,
+        sinkpad);
   } else {
-    gst_pad_stop_task (sinkpad);
+    wav->segment_running = FALSE;
+    return gst_pad_stop_task (sinkpad);
   }
-  gst_object_unref (wav);
-
-  return TRUE;
 };
 
 static GstStateChangeReturn
